﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Telerik.Windows.Documents.UI;
using Telerik.Windows.Documents.Model;
using Telerik.Windows.Controls;
using Telerik.Windows.Documents.FormatProviders.Html;
using System.IO;

namespace RGV_SL5_Print_PrintPreview
{
    public enum ExportFormat
    {
        Excel,
        Word,
        Pdf,
        Csv
    }

    public static class PrintAndExportExtensions
    {
        public static void Print(this RadGridView grid, PrintSettings settings)
        {
            var rtb = CreateRadRichTextBox(grid, settings);
            var window = new RadWindow() { Height = 0, Width = 0, Opacity = 0, Content = rtb };
            rtb.PrintCompleted += (s, e) => { window.Close(); };
            window.Show();

            rtb.Print("MyDocument", Telerik.Windows.Documents.UI.PrintMode.Native);
        }

        public static void PrintPreview(this RadGridView grid, PrintSettings settings)
        {
            var rtb = CreateRadRichTextBox(grid, settings);
            var window = CreatePreviewWindow(rtb);
            window.ShowDialog();
        }

        private static RadWindow CreatePreviewWindow(RadRichTextBox rtb)
        {
            var printButton = new RadButton()
            {
                Content = "Print",
                Margin = new Thickness(10, 0, 10, 0),
                FontWeight = FontWeights.Bold,
                Width = 80
            };

            printButton.Click += (s, e) =>
            {
                rtb.Print("MyDocument", Telerik.Windows.Documents.UI.PrintMode.Native);
            };

            var sp = new StackPanel() { Height = 26, Orientation = Orientation.Horizontal, Margin = new Thickness(10) };

            sp.Children.Add(new TextBlock() { Text = "Orientation:", Margin = new Thickness(10, 0, 3, 0), VerticalAlignment = VerticalAlignment.Center });

            var radComboBoxPageOrientation = new RadComboBox()
            {
                ItemsSource = new string[] { "Portrait", "Landscape" },
                SelectedIndex = 0
            };
            sp.Children.Add(radComboBoxPageOrientation);

            radComboBoxPageOrientation.SelectionChanged += (s, e) =>
            {
                rtb.ChangeSectionPageOrientation((PageOrientation)Enum.Parse(typeof(PageOrientation),
                    radComboBoxPageOrientation.Items[radComboBoxPageOrientation.SelectedIndex].ToString(), true));
            };

            sp.Children.Add(new TextBlock() { Text = "Size:", Margin = new Thickness(10, 0, 3, 0), VerticalAlignment = VerticalAlignment.Center });

            var radComboBoxPageSize = new RadComboBox()
            {
                ItemsSource = new string[] { "A0", "A1", "A2", "A3", "A4", "A5", "Letter" },
                Height = 25,
                SelectedIndex = 4,
            };
            sp.Children.Add(radComboBoxPageSize);

            radComboBoxPageSize.SelectionChanged += (s, e) =>
            {
                rtb.ChangeSectionPageSize(PaperTypeConverter.ToSize((PaperTypes)Enum.Parse(typeof(PaperTypes),
                    radComboBoxPageSize.Items[radComboBoxPageSize.SelectedIndex].ToString(), true)));
            };

            sp.Children.Add(printButton);

            var g = new Grid();
            g.RowDefinitions.Add(new RowDefinition() { Height = GridLength.Auto });
            g.RowDefinitions.Add(new RowDefinition());
            g.Children.Add(sp);
            g.Children.Add(rtb);

            Grid.SetRow(rtb, 1);

            return new RadWindow()
            {
                Content = g,
                Width = 900,
                Height = 600,
                Header = "Print Preview",
                WindowStartupLocation = Telerik.Windows.Controls.WindowStartupLocation.CenterScreen

            };
        }

        private static RadRichTextBox CreateRadRichTextBox(RadGridView grid, PrintSettings settings)
        {
            return new RadRichTextBox()
            {
                IsReadOnly = true,
                LayoutMode = DocumentLayoutMode.Paged,
                IsSelectionEnabled = false,
                IsSpellCheckingEnabled = false,
                Document = CreateDocument(grid, settings)
            };
        }

        private static RadDocument CreateDocument(RadGridView grid, PrintSettings settings)
        {
            RadDocument document = null;

            using (var stream = new MemoryStream())
            {
                grid.Export(stream, new GridViewExportOptions()
                {
                    Format = Telerik.Windows.Controls.ExportFormat.Html,
                    ShowColumnFooters = grid.ShowColumnFooters,
                    ShowColumnHeaders = grid.ShowColumnHeaders,
                    ShowGroupFooters = grid.ShowGroupFooters
                });

                stream.Position = 0;

                document = new HtmlFormatProvider().Import(stream);
            }

            return document;
        }
    }
}
